/*
Base64 - 1.1.0

Copyright (c) 2006 Steve Webster

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions: 

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

----------------------------------------------------------------------------------
Rewrite in Haxe by Clian VEYSSIRE (16/11/2008)

*/
package com.ikilote.utils;

import flash.utils.ByteArray;

class Base64 {
	
	private function new();
	
	static var BASE64_CHARS : String = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
 
	public static var version : String = "1.1.0";

	public static function encode(data : String ):String {
		// Convert string to ByteArray
		var bytes : ByteArray = new ByteArray();
		bytes.writeUTFBytes(data);
		
		// Return encoded ByteArray
		return encodeByteArray(bytes);
	}
	
	public static function encodeByteArray(data:ByteArray):String {
		// Initialise output
		var output : String = "";
		
		// Create data and output buffers
		var dataBuffer : Array<UInt>;
		var outputBuffer:Array<UInt> = new Array<UInt>();
		
		// Rewind ByteArray
		data.position = 0;
		
		// while there are still bytes to be processed
		while (data.bytesAvailable > 0) {
			// Create new data buffer and populate next 3 bytes from data
			dataBuffer = new Array();
			var i: UInt = 0;
			while(i < 3 && data.bytesAvailable > 0) {
				dataBuffer[i] = data.readUnsignedByte();
				i++;
			}
			
			// Convert to data buffer Base64 character positions and 
			// store in output buffer
			outputBuffer[0] = (dataBuffer[0] & 0xfc) >> 2;
			outputBuffer[1] = ((dataBuffer[0] & 0x03) << 4) | ((dataBuffer[1]) >> 4);
			outputBuffer[2] = ((dataBuffer[1] & 0x0f) << 2) | ((dataBuffer[2]) >> 6);
			outputBuffer[3] = dataBuffer[2] & 0x3f;
			
			// If data buffer was short (i.e not 3 characters) then set
			// end character indexes in data buffer to index of '=' symbol.
			// This is necessary because Base64 data is always a multiple of
			// 4 bytes and is basses with '=' symbols.
			var j : Int = dataBuffer.length;
			while (j < 3) {
				outputBuffer[j + 1] = 64;
				j++;
			}
			
			// Loop through output buffer and add Base64 characters to 
			// encoded data string for each character.
			var k : Int = 0;
			while (k < outputBuffer.length) {
				output += BASE64_CHARS.charAt(outputBuffer[k]);
				k++;
			}
		}
		
		// Return encoded data
		return output;
	}
	
	public static function decode(data:String):String {
		// Decode data to ByteArray
		var bytes:ByteArray = decodeToByteArray(data);
		
		// Convert to string and return
		return bytes.readUTFBytes(bytes.length);
	}
	
	public static function decodeToByteArray(data:String):ByteArray {
		// Initialise output ByteArray for decoded data
		var output:ByteArray = new ByteArray();
		
		// Create data and output buffers
		var dataBuffer : Array<UInt> = new Array<UInt>();
		var outputBuffer : Array<UInt> = new Array<UInt>();

		// While there are data bytes left to be processed
		var i : UInt = 0;
		while ( i < data.length) {
			// Populate data buffer with position of Base64 characters for
			// next 4 bytes from encoded data
			var j : UInt = 0;
			while (j < 4 && i + j < data.length) {
				dataBuffer[j] = BASE64_CHARS.indexOf(data.charAt(i + j));
				j++;
			}
			
			// Decode data buffer back into bytes
			outputBuffer[0] = (dataBuffer[0] << 2) + ((dataBuffer[1] & 0x30) >> 4);
			outputBuffer[1] = ((dataBuffer[1] & 0x0f) << 4) + ((dataBuffer[2] & 0x3c) >> 2);		
			outputBuffer[2] = ((dataBuffer[2] & 0x03) << 6) + dataBuffer[3];
			
			// Add all non-padded bytes in output buffer to decoded data
			var k : UInt = 0;
			while (k < outputBuffer.length) {
				if (dataBuffer[k+1] == 64) break;
				output.writeByte(outputBuffer[k]);
				k++;
			}
			i += 4;
		}
		
		// Rewind decoded data ByteArray
		output.position = 0;
		
		// Return decoded data
		return output;
	}
}
